=begin
(c) 2010 TIG & Refractive Software
All Rights Reserved.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING,WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Octane  - Exporter / Renderer Tool

Usage: File > 'Octane...', or type 'octane' in Ruby Console.

It opens the 'Octane' dialog, which exports SKP to OBJ format [for Octane], 
with other camera, sun info etc if desired, it can then 'render' an image 
using Octane, or make an 'animation'...

Version:

1022.
a 20100611 First beta release.
b 20100613 Second beta release. Camera location swap y/z fixed. 
           		Sun location swap y/z fixed.
           		Animations start-frame/end-frame/step from pull-down lists of 
          		 available Scenes [i.e. without those 'excluded from animation' 
          		 in the Scene Manager].
c 20100614 Help button added to open OctaneHelp.pdf in Octane folder.
         		  Animation Start/End/Step pull-downs linked and animation cmd 
          		 methods fixed.  'Step' not implement yet [always = 1].
          		 All settings now remembered with Model.
d 20100614 Step now implemented in animation.
e 20100615 UI messages improved. Animation typo error corrected.
            

Known Issues:

Selecting more GPUs that you have on your system can cause Octane to hang on startup.

The 'octane.cmd' and 'octane_anim.cmd' files [and also '_new_' files to make the 
ocs's if needed] and [suffixed '.command' on Mac] are not currently auto-deleted from 
the Project's Folder after execution - this is for debugging.
These files can currently be edited to view/change the contents/actions.

This tool is not yet well tested on Mac [as of version-1022.d].

Feedback welcome

TIG

=end
###
class Octane

    @@idset = %w(octane_render_binary project_path project_name native_unit_size resolution width height percentage export_camera active_camera lens_aperture_radius lens_aperture_radius_value focal_depth focal_depth_value motion_blur motion_blur_value export_sun_direction light_source gpu_devices gpu_device0 gpu_device1 gpu_device2 gpu_device3 image_output start_frame end_frame step pixel_samples pixs)

	@@dlg=UI::WebDialog.new(" Refractive Software 2010",false,"Octane",600,700,25,25,false) ###true=resizeable

	@@native_unit_size="meters"    
    
	@@resolution="false"
    @@percentage="100%"
    
    @@export_camera="true"
    @@active_camera="from <Current View>"
    @@lens_aperture_radius="false"
    @@lens_aperture_radius_value="1"
    @@focal_depth="false"
    @@focal_depth_value="5"
    @@motion_blur="false"
    @@motion_blur_value="Next"
    
    @@export_sun_direction="false"
    @@light_source="Sun from <Current View>"
    
    @@gpu_devices="false"
    @@gpu_device0="false"
    @@gpu_device1="false"
    @@gpu_device2="false"
    @@gpu_device3="false"

    @@image_output=""
    @@start_frame=nil
    @@end_frame=nil
    @@step=nil
    @@pixel_samples="Medium (256 s/px)"
    @@pixel_samples_array=["Very Low (8 s/px)", "Low (32 s/px)", "Medium (256 s/px)", "Medium/High (512 s/px)", "High (1024 s/px)", "Very High (4096 s/px)", "Custom"]
    @@pixs="256"

### end protected

  def initialize

    @msg=""
    @model=Sketchup.active_model
	@sel=@model.selection
    
    @dlg = @@dlg
    
	@octane_render_binary=Sketchup.read_default("Octane", "octane_render_binary", "").tr("\\\\","\\")
    
	@project_path=@model.get_attribute("Octane", "project_path", nil)
	@project_name=@model.get_attribute("Octane", "project_name", nil)
    
	@native_unit_size=@@native_unit_size
        
	@resolution=@model.get_attribute("Octane", "resolution", @@resolution)
    
    @width=@model.get_attribute("Octane", "width", @model.active_view.vpwidth.to_s)
    @height=@model.get_attribute("Octane", "height", @model.active_view.vpheight.to_s)
    @percentage=@model.get_attribute("Octane", "percentage", @@percentage)
    @percentages_array=["100%","75%","50%","25%","Reset"]
       
    @export_camera=@model.get_attribute("Octane", "export_camera", @@export_camera)
    @active_camera=@model.get_attribute("Octane", "active_camera", @@active_camera)
    @lens_aperture_radius=@model.get_attribute("Octane", "lens_aperture_radius", @@lens_aperture_radius)
    @lens_aperture_radius_value=@model.get_attribute("Octane", "lens_aperture_radius_value", @@lens_aperture_radius_value)
    @focal_depth=@model.get_attribute("Octane", "focal_depth", @@focal_depth)
    @focal_depth_value=@model.get_attribute("Octane", "focal_depth_value", @@focal_depth_value)
    @motion_blur=@model.get_attribute("Octane", "motion_blur", @@motion_blur)
    @motion_blur_value=@model.get_attribute("Octane", "motion_blur_value", @@motion_blur_value)
      
    @export_sun_direction=@model.get_attribute("Octane", "export_sun_direction", @@export_sun_direction)
    @light_source=@model.get_attribute("Octane", "light_source", @@light_source)
        
    @gpu_devices=@model.get_attribute("Octane", "gpu_devices", @@gpu_devices)
    @gpu_device0=@model.get_attribute("Octane", "gpu_device0", @@gpu_device0)
    @gpu_device1=@model.get_attribute("Octane", "gpu_device1", @@gpu_device1)
    @gpu_device2=@model.get_attribute("Octane", "gpu_device2", @@gpu_device2)
    @gpu_device3=@model.get_attribute("Octane", "gpu_device3", @@gpu_device3)
       
    @image_output=@model.get_attribute("Octane", "image_output", nil)
    
    self.get_scenes()
    
    @start_frame=@scenes[0]if @scenes[0]
    if @scenes.include?(@model.get_attribute("Octane", "start_frame", "()"))
      @start_frame=@model.get_attribute("Octane", "start_frame", nil)
    end#if
    @end_frame=@scenes[1]if @scenes[1]
    if @scenes.include?(@model.get_attribute("Octane", "end_frame", "()"))
      @end_frame=@model.get_attribute("Octane", "end_frame", nil)
    end#if
    @step=@model.get_attribute("Octane", "step", "1")
    
    @pixel_samples=@model.get_attribute("Octane", "pixel_samples", @@pixel_samples)
    @pixel_samples_array=@@pixel_samples_array
    @pixs=@model.get_attribute("Octane", "pixs", @@pixs)

	@dlg.add_action_callback("browse_octane_render_binary") {|d,p|self.browse_octane_render_binary()}
 	@dlg.add_action_callback("browse_project_path") {|d,p|self.browse_project_path()}
    @dlg.add_action_callback("use_existing_project") {|d,p|self.use_existing_project()}
    @dlg.add_action_callback("update_project_name") {|d,p|self.update_project_name()}

    @dlg.add_action_callback("resolution"){|d,p|self.resolution(p)}
    @dlg.add_action_callback("percentage_changed"){|d,p|self.percentage_changed()}
        
    @dlg.add_action_callback("export_camera"){|d,p|self.export_camera(p)}
    @dlg.add_action_callback("set_camera"){|d,p|self.set_camera()}
    @dlg.add_action_callback("lens_aperture_radius"){|d,p|self.lens_aperture_radius(p)}
    @dlg.add_action_callback("focal_depth"){|d,p|self.focal_depth(p)}
    @dlg.add_action_callback("motion_blur"){|d,p|self.motion_blur(p)}
      
    @dlg.add_action_callback("export_sun_direction"){|d,p|self.export_sun_direction(p)}
        
    @dlg.add_action_callback("gpu_devices"){|d,p|self.gpu_devices(p)}
        
	@dlg.add_action_callback("export"){|d,p|self.export()}
	@dlg.add_action_callback("render"){|d,p|self.render()}
        
    @dlg.add_action_callback("select_image_output"){|d,p|self.select_image_output()}
    @dlg.add_action_callback("render_animation"){|d,p|self.render_animation()}
    
    @dlg.add_action_callback("update_start"){|d,p|self.update_start()}
    @dlg.add_action_callback("update_end"){|d,p|self.update_end()}
    @dlg.add_action_callback("update_step"){|d,p|self.update_step()}
    
    @dlg.add_action_callback("update_pixs"){|d,p|self.update_pixs()}
    @dlg.add_action_callback("update_pixs_txt"){|d,p|self.update_pixs_txt()}
    
    @dlg.add_action_callback("help"){|d,p|self.help()}
    
    @dlg.add_action_callback("set_button"){|d,p|self.set_button()}
        
	@dlg.add_action_callback("populate"){|d,p|self.populate()}
    ### 
  end

  def activate
    @pc=false
    @pc=true if [PLATFORM].grep(/mswin/)==[PLATFORM]
    ### check on exe
    if @octane_render_binary==""
      UI.messagebox("Octane:\nThe 'Octane Render Binary' must be set...")
      self.browse_octane_render_binary()
    elsif not File.exist?(@octane_render_binary.tr("\\","/"))
      UI.messagebox("Octane:\nThe 'Octane Render Binary' is not valid...")
      self.browse_octane_render_binary()
    end#if
    @octane_render_binary.tr("/","\\")if @pc
    ### set up project
    @path=@model.path
    if @path==""
       UI.messagebox("Octane:\nYou must save this new Sketchup model before using this tool!")
       self.deactivate()
       return nil
    end#if
    ###
    project_path=File.dirname(@path)
    project_name=@model.title.tr(" ","_")
    @project_path=project_path if not @project_path
    @project_name=project_name if not @project_name
    @project_name.tr!(" ","_")
    image_output=@project_path if not @image_output
    @image_name="img00001.png"
    @image_output=File.join(image_output, @image_name) if not @image_output###
    ###
    @project_path.tr!("/","\\")if @pc
    @image_output.tr!("/","\\")if @pc
    ### now do dialog
    @first_time=true
    self.dialog()
  end#activate
  
  def deactivate(view=nil)
    self.saveIDvalues()
    @dlg.close() if @dlg.visible? ### ???
    view.invalidate if view
    Sketchup.send_action("selectSelectionTool:")
    return nil
  end#deactivate
  
  def dialog()
    default_html=File.dirname(__FILE__)+"/OctaneDialog.html"
    ### possible locale language based dialogs alternatives
	locale_html=File.dirname(__FILE__)+"/OctaneDialog-"+Sketchup.get_locale+".html"
	@dlg=@@dlg unless @dlg
	if File.exist?(locale_html)
      @dlg.set_file(locale_html)
    else
      @dlg.set_file(default_html)
    end#if
	@dlg.show
	sleep(0.333)
  end#dialog
  
  def populate()
    ###return nil
    #set octane_render_binary path
    octane_render_binary=@octane_render_binary.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"octane_render_binary\").setAttribute(\"value\",\""+octane_render_binary+"\");")
    @dlg.execute_script(js)
    #set project_path
    project_path=@project_path.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"project_path\").setAttribute(\"value\",\""+project_path+"\");")
    @dlg.execute_script(js)
    #set project_name
    project_name=@project_name
    js=("document.getElementById(\"project_name\").setAttribute(\"value\",\""+project_name+"\");")
    @dlg.execute_script(js)
    #set extg native_unit_size
    units=@model.options["UnitsOptions"]["LengthUnit"]
    case units
      when 1
        @native_unit_size="feet"
      when 2
        @native_unit_size="millimeters"
      when 3
        @native_unit_size="centimeters"
      when 4
        @native_unit_size="meters"
      else ###0
        @native_unit_size="inches"
    end#case
    #set native_units
    units_array=["inches","feet","meters","centimeters","millimeters"]
	units_array.each_with_index do |unit,i|
		js="document.getElementById(\"native_unit_size\")[#{i}] = new Option('#{unit}','#{unit}'"
		js+=",true,true" if @native_unit_size==unit ### SELECTED UNITS
		js+=");"
		@dlg.execute_script(js)
	end#do
    #set resolution
    if @resolution=="true"
      js=("document.getElementById(\"resolution\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"resolution\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    # h/w
    if @first_time
      #@width=@model.active_view.vpwidth.to_s
      js=("document.getElementById(\"width\").setAttribute(\"value\",\""+@width+"\");")
      @dlg.execute_script(js)
      #@height=@model.active_view.vpheight.to_s
      js=("document.getElementById(\"height\").setAttribute(\"value\",\""+@height+"\");")
      @dlg.execute_script(js)
    end#if
    @width=@dlg.get_element_value("width")
    @height=@dlg.get_element_value("height")
    if @width.to_i==0 or @height.to_i==0
      @width=@model.active_view.vpwidth.to_s
      @height=@model.active_view.vpheight.to_s
      @percentage="100%"
    end#if
    #set h/w
    js=("document.getElementById(\"width\").setAttribute(\"value\",\""+@width+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"height\").setAttribute(\"value\",\""+@height+"\");")
    @dlg.execute_script(js)
    #set percentage
    @percentage=@dlg.get_element_value("percentage") if not @first_time
	@percentages_array.each_with_index do |p,i|
		js="document.getElementById(\"percentage\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @percentage==p ### SELECTED Percent
		js+=");"
		@dlg.execute_script(js)
	end#do
    @percentage=@dlg.get_element_value("percentage")
    ### always have export_camera set and disabled
    #set export_camera could be omitted as we must always export a camera to render ?
    if @export_camera=="true"
      js=("document.getElementById(\"export_camera\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_camera\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    #set active_camera
    @active_camera=@dlg.get_element_value("active_camera") if not @first_time
    @cameras_array=["from <Current View>"]
    ### check for scenes
    @pages=[]
    @model.pages.each{|page|
      name=page.name
      #name=page.label if name != page.label ### in anim only ?
      @pages<< name
    }
    @cameras_array<< "from <Current Scene>" if @pages[0]
    @cameras_array=@cameras_array+@pages
    if not @cameras_array.include?(@active_camera)
      @active_camera=@cameras_array[0]
    end#if
    ###
	@cameras_array.each_with_index do |p,i|
		js="document.getElementById(\"active_camera\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @active_camera==p ### SELECTED camera
		js+=");"
		@dlg.execute_script(js)
	end#do
    @active_camera=@dlg.get_element_value("active_camera")
    #set lens_aperture_radius
    if @lens_aperture_radius=="true"
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").setAttribute(\"disabled\",false);")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").setAttribute(\"disabled\",true);")
      #@dlg.execute_script(js)
    end#if
    #set lens_aperture_radius
    js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"value\",\""+@lens_aperture_radius_value+"\");")
    @dlg.execute_script(js)
    #set focal_depth
    if @focal_depth=="true"
      js=("document.getElementById(\"focal_depth\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").setAttribute(\"disabled\",false);")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"focal_depth\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").setAttribute(\"disabled\",true);")
      #@dlg.execute_script(js)
    end#if
    #set focal_depth_value
    js=("document.getElementById(\"focal_depth_value\").setAttribute(\"value\",\""+@focal_depth_value+"\");")
    @dlg.execute_script(js)
    #set motion_blur
    if @motion_blur=="true"
      js=("document.getElementById(\"motion_blur\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
     js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"motion_blur\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    #set motion_blur_value
    @motion_blur_value=@dlg.get_element_value("motion_blur_value") if not @first_time
    motion_blur_array=["Next","Previous"]
	motion_blur_array.each_with_index do |p,i|
		js="document.getElementById(\"motion_blur_value\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @motion_blur_value==p ### SELECTED motion_blur_value
		js+=");"
		@dlg.execute_script(js)
	end#do
    @motion_blur_value=@dlg.get_element_value("motion_blur_value")
    #set export_sun_direction
    if @export_sun_direction=="true"
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    #set light_sources
    @light_source=@dlg.get_element_value("light_source")
    @lights_array=["Sun from <Current View>"]
    ### check for scenes
    @pages=[]
    @model.pages.each{|page|
      name=page.name
      ###name=page.label if name != page.label
      @pages<< name
    }
    @lights_array<< "Sun from <Current Scene>" if @pages[0]
    @lights_array=@lights_array+@pages
    if not @lights_array.include?(@light_source)
      @light_source=@lights_array[0]
    end#if
    ###
	@lights_array.each_with_index do |p,i|
		js="document.getElementById(\"light_source\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @light_source==p ### SELECTED light
		js+=");"
		@dlg.execute_script(js)
	end#do
    @light_source=@dlg.get_element_value("active_camera")
    #set gpu_devices
    if @gpu_devices=="true"
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    #set image_output
    image_output=@image_output.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"image_output\").setAttribute(\"value\",\""+image_output+"\");")
    @dlg.execute_script(js)
    #set start_frame etc
    if @start_frame
      @start_frame=@dlg.get_element_value("start_frame") if not @first_time
      self.update_start()
      self.update_end()
      self.update_step()
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###
    if @end_frame
      self.update_end()
      self.update_step()
      @end_frame=@dlg.get_element_value("end_frame")
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###
    self.update_step()
    if @step
      @step=@dlg.get_element_value("step")
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###
    #set pixrelsamples
    @pixel_samples=@dlg.get_element_value("pixel_samples") if not @first_time
    pixel_samples_array=@pixel_samples_array
	pixel_samples_array.each_with_index do |p,i|
		js="document.getElementById(\"pixel_samples\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @pixel_samples==p ### SELECTED pixel_samples
		js+=");"
		@dlg.execute_script(js)
	end#do
    @pixel_samples=@dlg.get_element_value("pixel_samples")
    js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",false);")
    @dlg.execute_script(js)
    ### pixs
    pixsa=@dlg.get_element_value("pixel_samples")
    if pixsa=~/Custom/
      pixs=@pixs
    else
      pixs=pixsa.split("(")[1].to_i.to_s
    end#if    
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pixs+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",false);")
    @dlg.execute_script(js)
    @pixs==@dlg.get_element_value("pixs")
    ### trap for start_frame etc
    if not @start_frame or not @end_frame or not @step
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###
    @first_time=false
    ###
  end#populate  
  
  def browse_octane_render_binary()
    Sketchup.set_status_text("Octane: Browse for 'Octane Render Binary'...")
    @octane_render_binary=@dlg.get_element_value("octane_render_binary")if @dlg.visible?
    ### browse for file
    binary=UI.openpanel("Choose the 'Refractive Software' - 'Octane Render Binary'...", nil, "*.exe")
    if not binary and not File.exist?(@octane_render_binary.tr("\\","/"))
      UI.messagebox("Octane:\nCannot continue without a valid 'Octane Render Binary'!")
      self.deactivate()
      return nil
    end#if
    if binary
      binary.tr!("/","\\")if @pc ### fix for PC openpanel - file seperator on PC = \\, but Mac = /
      @octane_render_binary=binary
      octane_render_binary=binary.gsub(/\\/){'\\\\'}
      Sketchup.write_default("Octane", "octane_render_binary", octane_render_binary)
    end#if
    ###
    octane_render_binary=@octane_render_binary.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"octane_render_binary\").setAttribute(\"value\",\""+octane_render_binary+"\");")
    @dlg.execute_script(js)
    Sketchup.set_status_text("Octane:")
  end#browse_octane_render_binary
  
  def browse_project_path()
    Sketchup.set_status_text("Octane: Browse 'Project Path'...")
    ###
    @project_path=@dlg.get_element_value("project_path")if @dlg.visible?
    ### browse for folder
    project=UI.openpanel("Choose any file in a folder to select the 'Project Folder'...", @project_path)
    if not project and not File.exist?(@project_path.tr("\\","/"))
      UI.messagebox("Octane:\nCannot continue without a valid 'Project Path'!")
      self.deactivate()
      return nil
    end#if
    if project
      @project_path=File.dirname(project.tr("\\","/"))
      @project_path.tr!("/","\\")if @pc
      @model.set_attribute("Octane", "project_path", @project_path)
    end#if
    project_path=@project_path.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"project_path\").setAttribute(\"value\",\""+project_path+"\");")
    @dlg.execute_script(js)
    ###self.populate()
    Sketchup.set_status_text("Octane:")
  end#browse_project_path
  
  def use_existing_project()
    Sketchup.set_status_text("Octane: Use Existing Project...")
    ### browse for file
    project=UI.openpanel("Choose 'Existing Project'...", nil, "*.ocs")
    if not project and not File.exist?(@project_path.tr("\\","/"))
      UI.messagebox("Octane:\nCannot continue without a valid 'Project Path'!")
      self.deactivate()
      return nil
    end#if
    if project
      @project_path=File.dirname(project.tr("\\","/"))
      @project_path.tr!("/","\\")if @pc
      @model.set_attribute("Octane", "project_path", @project_path)
      @project_name=File.basename(project, ".ocs")
      @project_name.tr!(" ","_")
      @model.set_attribute("Octane", "project_name", @project_name)
    end#if
    project_name=@project_name
    js=("document.getElementById(\"project_name\").setAttribute(\"value\",\""+project_name+"\");")
    @dlg.execute_script(js)
    ###self.populate()
    Sketchup.set_status_text("Octane:")
  end#use_existing_project
  
  def update_project_name()
    @project_name=@dlg.get_element_value("project_name")
    @model.set_attribute("Octane", "project_name", @project_name)
  end#update_project_name
  
  def resolution(p)
    @resolution=p
    if @resolution=="true"
      js=("document.getElementById(\"resolution\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"resolution\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###self.populate()
  end#resolution
  
  def percentage_changed()
    @percentage=@dlg.get_element_value("percentage")
    ### check if w/h==0 and reset if do...
    if @percentage=="Reset"
      @width=@model.active_view.vpwidth.to_s
      @height=@model.active_view.vpheight.to_s
      @percentage="100%"
    end#if
    # RE-set percentage
	@percentages_array.each_with_index do |p,i|
		js="document.getElementById(\"percentage\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @percentage==p ### SELECTED Percent
		js+=");"
		@dlg.execute_script(js)
	end#do
    @percentage=@dlg.get_element_value("percentage")
    ### update width/height by %
    @width=(@width.to_i * @percentage.to_i / 100).to_i.to_s
    js=("document.getElementById(\"width\").setAttribute(\"value\",\""+@width+"\");")
    @dlg.execute_script(js)
    @height=(@height.to_i * @percentage.to_i / 100).to_i.to_s
    js=("document.getElementById(\"height\").setAttribute(\"value\",\""+@height+"\");")
    @dlg.execute_script(js)
    ###self.populate()
  end
  
  def export_camera(p)
    @export_camera=p
    @active_camera=@dlg.get_element_value("active_camera")
    if @export_camera=="true"
        js=("document.getElementById(\"export_camera\").setAttribute(\"checked\",true);")
        @dlg.execute_script(js)
        js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",false);")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",false);")
        @dlg.execute_script(js)
    else
        js=("document.getElementById(\"export_camera\").setAttribute(\"checked\",false);")
        @dlg.execute_script(js)
        js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",true);")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",true);")
        @dlg.execute_script(js)
    end#if
    ###self.populate()
  end#export_camera
  
  def set_camera()
    @active_camera=@dlg.get_element_value("active_camera")
  end#set_camera
  
  def lens_aperture_radius(p)
    @lens_aperture_radius=p
    @lens_aperture_radius_value=@dlg.get_element_value("lens_aperture_radius_value")
    if @lens_aperture_radius=="true"
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").setAttribute(\"disabled\",false);")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").setAttribute(\"disabled\",true);")
      #@dlg.execute_script(js)
    end#if
    ###self.populate()
  end#lens_aperture_radius
  
  def focal_depth(p)
    @focal_depth=p
    @focal_depth_value=@dlg.get_element_value("focal_depth_value")
    if @focal_depth=="true"
      js=("document.getElementById(\"focal_depth\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").setAttribute(\"disabled\",false);")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"focal_depth\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").setAttribute(\"disabled\",true);")
      #@dlg.execute_script(js)
    end#if
    ###self.populate()
  end#lens_aperture_radius
  
  def motion_blur(p)
    @motion_blur=p
    @motion_blur_value=@dlg.get_element_value("motion_blur_value")
    if @motion_blur=="true"
      js=("document.getElementById(\"motion_blur\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"motion_blur\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###self.populate()
  end#lens_aperture_radius

  def export_sun_direction(p)
    @export_sun_direction=p
    @light_source=@dlg.get_element_value("light_source")
    if @export_sun_direction=="true"
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###self.populate()
  end#export_sun_direction
  
  def gpu_devices(p)
    @gpu_devices=p
    @gpu_device0=@dlg.get_element_value("gpu_device0")
    @gpu_device1=@dlg.get_element_value("gpu_device1")
    @gpu_device2=@dlg.get_element_value("gpu_device2")
    @gpu_device3=@dlg.get_element_value("gpu_device3")
    if @gpu_devices=="true"
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",false);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",true);")
      @dlg.execute_script(js)
    end#if
    ###self.populate()
  end#gpu_devices
  
  def export(anim=false)
    Sketchup.set_status_text("Octane: Exporting OBJ & MTL Files...")
    self.getIDvalues()
    ### check for validity
    if @project_path=="" or not File.exist?(@project_path)
      UI.messagebox("Octane:\nProject Path is not valid.")
      return nil
    end#if
    if @project_name=="" or @project_name=~/ /
      UI.messagebox("Octane:\nProject Name is not valid.")
      return nil
    end#if
    ###
    self.run_exporter(anim)
    ###
  end#export
  
  def render()
    self.export()
    ###
	Sketchup.set_status_text("Octane: Render...")
    ### make ocs if not there
    prp=@project_path.tr("/","\\")
    prn=@project_name
    ocs=prp+"\\"+prn+".ocs" ###
    obj=prp+"\\"+prn+".obj" ###
    obn=prn+".obj" ###
    if not File.exist?(ocs)
      cline=""
      cline=cline+"@echo off\n" if @pc
      orb=@octane_render_binary
      cline=cline+"@" if @pc
      cline=cline+"\""+orb+"\" -e -q -s 1"
      cline=cline+" -n \""+ocs+"\""
      cline=cline+" -m \""+obn+"\""
      cline=cline+" -l \""+obj+"\"\n"
      cline=cline+"@exit\n" if @pc
     if @pc
      ### make cmd file
      cmd_path=@project_path.tr("\\","/")+"/octane_new.cmd"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
     else ### it's a Mac
      #UI.messagebox("Mac version is in beta testing...")
      cline.tr("\\","/")### force unix friendly separators
      cline.gsub(/\"/){'\\"'}### escape " as \"
      cmd_path=@project_path.tr("\\","/")+"/octane_new.command"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      File.chmod(1411, cmd_path) ### 777 in octal is 1411 = execuatble
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
      ###
     end#if
     sleep 4
    end#if
    ### set variables for command-line
    cline=""
    cline=cline+"@echo off\n" if @pc
    ###
    orb=@octane_render_binary
    cline=cline+"@" if @pc
    cline=cline+"\""+orb+"\" "
    cline=cline+"-s 4096 "
    ### so continues to render to very high till stopped
    ###
    prp=@project_path.tr("/","\\")
    prn=@project_name
    ocs=prp+"\\"+prn+".ocs"
    ###
    ona=@obj_name
    cline=cline+"-m \""+ona+"\""
    ###
    ofi=@obj_filepath.tr("/","\\")
    #cline=cline+" -l \""+ofi+"\"" already linked when made new ?
    cline=cline+" -r \""+ofi+"\""
    ###
    if @resolution=="true"
      wi=@width
      he=@height
      cline=cline+" --film-width " +wi.to_s
      cline=cline+" --film-height "+he.to_s
    else
      wi=nil
      he=nil
    end#if
    ### active camera
  if @export_camera=="true"
    cams=@cameras_array
    acc=@model.active_view.camera
    pages=@model.pages
    case @active_camera
      when "from <Current View>"
        acc=@model.active_view.camera
      when "from <Current Scene>"
        acc=@model.pages.selected_page.camera
      else
        pages.each{|p|acc=p.camera if p.name==@active_camera}
    end#case    
    posa=acc.eye.to_a
    pos=[0,0,0]
    pos.x=posa.x.to_m.to_f
    pos.y=posa.z.to_m.to_f      ###
    pos.z=posa.y.to_m.to_f * -1 ###
    targeta=acc.target.to_a
    target=[0,0,0]
    target.x=targeta.x.to_m.to_f
    target.y=targeta.z.to_m.to_f        ###
    target.z=targeta.y.to_m.to_f * -1   ###
    upa=acc.up.to_a
    up=[0,0,0]
    up.x=upa.x.to_m.to_f
    up.y=upa.z.to_m.to_f        ###
    up.z=upa.y.to_m.to_f * -1   ###
    ###
    cline=cline+" --cam-pos-x "+pos.x.to_s
    cline=cline+" --cam-pos-y "+pos.y.to_s
    cline=cline+" --cam-pos-z "+pos.z.to_s
    ###
    cline=cline+" --cam-target-x "+target.x.to_s
    cline=cline+" --cam-target-y "+target.y.to_s
    cline=cline+" --cam-target-z "+target.z.to_s
    ###
    cline=cline+" --cam-up-x "+up.x.to_s
    cline=cline+" --cam-up-y "+up.y.to_s
    cline=cline+" --cam-up-z "+up.z.to_s
    ###
    fol=acc.focal_length
    fovV=acc.fov
    wid=@width.to_f
    hei=@height.to_f
    if acc.image_width != 0
      wid=acc.image_width
      if acc.aspect_ratio != 0
        aro=acc.aspect_ratio
      else
        aro=wid/hei
      end#if
      hei=wid/aro
    end#if
    h=2*fol*Math::tan(fovV.degrees/2)
    w=h*wid/hei
    fov=(2*Math::atan(w/(2*fol))).radians
    cline=cline+" --cam-fov "+fov.to_s
    ###
  end#if export_camera
    ###
    if @lens_aperture_radius=="true"
      apr=@lens_aperture_radius_value
      cline=cline+" --cam-aperture "+apr.to_s
    else
      apr=nil
    end#if
    if @focal_depth=="true"
      foc=@focal_depth_value
      cline=cline+" --cam-focaldepth "+foc.to_s
    end#if
    ###
    if @motion_blur=="true"
      mob=@motion_blur_value
    else
      mob=""
    end#if
    mbc=nil
    case mob
      when "Next"
        if @active_camera=="from <Current View>"
          mbc=nil
        elsif @active_camera=="from <Current Scene>"
          pages=@model.pages.to_a
          cac=@model.pages.selected_page
          cur=pages.index(cac)
          nxt=cur+1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[0].camera
          end#if
        else
          pages=@model.pages.to_a
          cac=pages[0]
          pages.each{|p|cac=p if p.name==@active_camera}
          cur=pages.index(cac)
          nxt=cur+1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[0].camera
          end#if
        end#if
      when "Previous"
        if @active_camera=="from <Current View>"
          mbc=nil
        elsif @active_camera=="from <Current Scene>"
          pages=@model.pages.to_a
          cac=@model.pages.selected_page
          cur=pages.index(cac)
          nxt=cur-1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[-1].camera
          end#if
        else
          pages=@model.pages.to_a
          cac=pages[0]
          pages.each{|p|cac=p if p.name==@active_camera}
          cur=pages.index(cac)
          nxt=cur-1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[-1].camera
          end#if
        end#if
      else
        mbc=nil
    end#case
    if mbc
      mposa=mbc.eye.to_a
      mpos=mposa
      mpos.x=mposa.x.to_m.to_f
      mpos.y=mposa.z.to_m.to_f
      mpos.z=mposa.y.to_m.to_f * -1
      mtargeta=mbc.target.to_a
      mtarget=mtargeta
      mtarget.x=mtargeta.x.to_m.to_f
      mtarget.y=mtargeta.z.to_m.to_f
      mtarget.z=mtargeta.y.to_m.to_f * -1
      mupa=mbc.up.to_a
      mup=mupa
      mup.x=mupa.x.to_m.to_f
      mup.y=mupa.z.to_m.to_f
      mup.z=mupa.y.to_m.to_f * -1
      ###
      cline=cline+" --cam-motion-pos-x "+mpos.x.to_s
      cline=cline+" --cam-motion-pos-y "+mpos.y.to_s
      cline=cline+" --cam-motion-pos-z "+mpos.z.to_s
      ###
      cline=cline+" --cam-motion-target-x "+mtarget.x.to_s
      cline=cline+" --cam-motion-target-y "+mtarget.y.to_s
      cline=cline+" --cam-motion-target-z "+mtarget.z.to_s
      ###
      cline=cline+" --cam-motion-up-x "+mup.x.to_s
      cline=cline+" --cam-motion-up-y "+mup.y.to_s
      cline=cline+" --cam-motion-up-z "+mup.z.to_s
      ###
    end#if
    ### sun etc ?
    if @export_sun_direction=="true"
      pages=@model.pages.to_a
      case @light_source
        when "Sun from <Current View>"
          sunv=@model.shadow_info["SunDirection"]
        when "Sun from <Current Scene>"
          sunv=@model.pages.selected_page.shadow_info["SunDirection"]
        else
          spa=nil
          pages.each{|p|spa=p if p.name==@light_source}
          sunv=spa.shadow_info["SunDirection"]
      end#case
      suna=sunv.to_a
      sun=[0,0,0]
      sun.x=suna.x.to_f
      sun.y=suna.z.to_f
      sun.z=suna.y.to_f * -1
      cline=cline+" --daylight-sundir-x "+sun.x.to_s
      cline=cline+" --daylight-sundir-y "+sun.y.to_s
      cline=cline+" --daylight-sundir-z "+sun.z.to_s
    end#if
    ### GPUs ?
    if @gpu_devices=="true"
      if @gpu_device0=="true"
        cline=cline+" -g 0"
      end#if
      if @gpu_device1=="true"
        cline=cline+" -g 1"
      end#if
      if @gpu_device2=="true"
        cline=cline+" -g 2"
      end#if
      if @gpu_device3=="true"
        cline=cline+" -g 3"
      end#if
    end#if
    ###
    cline=cline+" \""+ocs+"\"\n"
    cline=cline+"@exit\n" if @pc
    ###
    if @pc
      ### make cmd file
      cmd_path=@project_path.tr("\\","/")+"/octane.cmd"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
    else ### it's a Mac
      #UI.messagebox("Mac version is in beta testing...")
      cline.tr("\\","/")### force unix friendly separators
      cline.gsub(/\"/){'\\"'}### escape " as \"
      cmd_path=@project_path.tr("\\","/")+"/octane.command"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      File.chmod(1411, cmd_path) ### 777 in octal is 1411 = execuatble
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
      ###
    end#if
    ###
  end#render
  
  def select_image_output()
    Sketchup.set_status_text("Octane: Select Image Output...")
    ### browse for folder
    @image_output=@dlg.get_element_value("image_output")
    ###
    image_output=UI.openpanel("Choose the 'Image Output' File [.PNG] or Any File to Select its Folder...")
    if not image_output
      return nil
    end#if
    if image_output
      image_output.tr!("/","\\")if @pc ### fix for PC openpanel - file seperator on PC = \\, but Mac = /
      if File.extname(image_output).downcase==".png"
        image_output=File.dirname(image_output)
        image_name=File.basename(image_output)
      else
        image_output=File.dirname(image_output)
        image_name=@image_name
      end#if
      image_out=File.join(image_output, image_name)
      image_out.tr!("/","\\")if @pc
      @image_output=image_out
      @model.set_attribute("Octane", "image_output", @image_output)
    end#if
    ###
    image_out=@image_output.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"image_output\").setAttribute(\"value\",\""+image_out+"\");")
    @dlg.execute_script(js)
    ###self.populate()
    Sketchup.set_status_text("Octane:")
  end#select_image_output
  
  def update_pixs()
    pixsa=@dlg.get_element_value("pixel_samples")
    if pixsa=~/Custom/
      pixs=@pixs
    else
      pixs=pixsa.split("(")[1].to_i.to_s
    end#if    
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pixs+"\");")
    @dlg.execute_script(js)
    @pixs=@dlg.get_element_value("pixs")
  end#update_pixs
  
  def update_pixs_txt()
    pixs=@dlg.get_element_value("pixs")
    pix=pixs.to_i.to_s
    ### update list too
    tester="Custom"
    pixel_samples_array=@pixel_samples_array
    (pixel_samples_array-[tester]).each{|txt|
      if pix == txt.split("(")[1].to_i.to_s
        tester=txt
        break
      end#if
    }
    pixel_samples_array.each_with_index do |p,i|
    js="document.getElementById(\"pixel_samples\")[#{i}] = new Option('#{p}','#{p}'"
	js+=",true,true" if p==tester ### SELECTED pixel_samples
	js+=");"
	@dlg.execute_script(js)
    @pixel_samples=@dlg.get_element_value("pixel_samples")
    end#if
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pix+"\");")
    @dlg.execute_script(js)
    @pixs=@dlg.get_element_value("pixs")
  end#update_pixs_txt
  
  def update_start()
    if not @start_frame or @start_frame=="" or not @scenes.include?(@start_frame)
      @start_frame=@scenes[0]
    else
      @start_frame=@dlg.get_element_value("start_frame")
    end#if
    start_frame_array=@scenes[0..-2] ### not the last one
	start_frame_array.each_with_index do |p,i|
		js="document.getElementById(\"start_frame\")[#{i}] = new Option('#{p}','#{p}'"
        js+=",true,true" if @start_frame==p ### SELECTED start_frame
		js+=");"
		@dlg.execute_script(js)
	end#do
    @start_frame=@dlg.get_element_value("start_frame")
  end#update_start
  
  def update_end()
    if not @end_frame or @end_frame=="" or not @scenes.include?(@end_frame)
      @end_frame=@scenes[0]
    else
      @end_frame=@dlg.get_element_value("end_frame")
    end#if
    end_frame_array=@scenes[1..-1] ### not the first one
	end_frame_array.each_with_index do |p,i|
		js="document.getElementById(\"end_frame\")[#{i}] = new Option('#{p}','#{p}'"
        js+=",true,true" if @end_frame==p ### SELECTED end_frame
		js+=");"
		@dlg.execute_script(js)
	end#do
    @end_frame=@dlg.get_element_value("end_frame")
  end#update_end
  
  def update_step()
    if not @step or @step==""
      @step="1"
    else
      begin
        @step=@dlg.get_element_value("step")
      rescue
        @step="1"
      end
    end#if
    ### calculate step list
    step_array=[]
    if @scenes[1]
      @scenes[1..-1].length.times{|i|step_array << (i+1).to_s}
    end#if
    if not step_array.include?(@step) or not step_array[0]
      @step="1"
    end#if
    step_array.each_with_index do |p,i|
	  js="document.getElementById(\"step\")[#{i}] = new Option('#{p}','#{p}'"
      js+=",true,true" if @step==p ### SELECTED step
	  js+=");"
	  @dlg.execute_script(js)
	end#do
    @step=@dlg.get_element_value("step")
  end#update_step
  
  def get_scenes()### only 'anim' scenes
    @all_pages=@model.pages.to_a
    @scenes=[]; @all_pages.each{|p|@scenes<< p.name if p.name==p.label}
  end#get_scenes
  
  def render_animation()
    return nil if not @start_frame or @start_frame=="" or not @scenes.flatten.include?(@start_frame)
    self.set_button(true)
    js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",true);")
    @dlg.execute_script(js)
    ###
    anim=true
    self.export(anim) ### also gets IDs
    Sketchup.set_status_text("Octane: Rendering Animation [To Abort - Close the Separate 'Cmd' Window]...")
    ### set up new vars for anim
    imgout=@image_output
    imgdiry=File.dirname(imgout)
    imgextn=File.extname(imgout)
    imgname=File.basename(imgout, imgextn)
    ###
    startf=@start_frame
    endf=@end_frame
    step=@step.to_i
    ###
    pixs=@pixs
    ###
    opu=@image_output
    opudiry=File.dirname(opu)
    opuextn=File.extname(opu)
    opuname=File.basename(opu, opuextn)
    opu=File.join(opudiry, opuname)
    opu.tr!("/","\\")
    ###
    pages=@model.pages
    cameras=[]; pages.each{|p|cameras << p.camera}
    ###
    scenes=@scenes
    s_indx=scenes.index(startf)
    e_indx=scenes.index(endf)
    anim_scenes=scenes[s_indx..e_indx]
    ### adjust array for step
    step_scenes=[]
    0.step(anim_scenes.length-1, step){|i|step_scenes << anim_scenes[i]}
    anim_scenes=step_scenes
    ### 
    ### make ocs if not there
    prp=@project_path.tr("/","\\")
    prn=@project_name
    ocs=prp+"\\"+prn+"_anim.ocs" ###
    obj=prp+"\\"+prn+"_anim.obj" ###
    obn=prn+"._anim.obj" ###
    if not File.exist?(ocs)
      cline=""
      cline=cline+"@echo off\n" if @pc
      orb=@octane_render_binary
      cline=cline+"@" if @pc
      cline=cline+"\""+orb+"\" -e -q -s 1"
      cline=cline+" -n \""+ocs+"\""
      cline=cline+" -m \""+obn+"\""
      cline=cline+" -l \""+obj+"\"\n"
      cline=cline+"@exit\n" if @pc
     if @pc
      ### make cmd file
      cmd_path=@project_path.tr("\\","/")+"/octane_anim_new.cmd"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
     else ### it's a Mac
      #UI.messagebox("Mac version is in beta testing...")
      cline.tr("\\","/")### force unix friendly separators
      cline.gsub(/\"/){'\\"'}### escape " as \"
      cmd_path=@project_path.tr("\\","/")+"/octane_anim_new.command"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      File.chmod(1411, cmd_path) ### 777 in octal is 1411 = execuatble
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
      ###
     end#if
     sleep 4
    end#if
    ###
    ### set loop for frames >>>>>>>>>>>>>>>>>>>
    ### set variables for command-line
    cline=""
    cline=cline+"@echo off\n" if @pc
    cline=cline+"title Animation\n" if @pc
    ###
  anim_scenes.each{|scene_name|
    scene=nil
    @model.pages.each{|p|
      if p.name==scene_name
        scene=p
        break
      end#if
    }
    next if not scene
    ### set variables for command-line
    ###
    orb=@octane_render_binary
    cline=cline+"@" if @pc
    cline=cline+"\""+orb+"\" -e -q"
    ###
    prp=@project_path.tr("/","\\")
    prn=@project_name
    ocs=prp+"\\"+prn+"_anim.ocs"
    ###
    ### here we add the opu etc
    opux=opu+opuextn
    ###
    cline=cline+" -o "+opux
    ###
    sam=@pixs
    cline=cline+" -s "+sam
    ###
    ona=@obj_name
    cline=cline+" -m \""+ona+"\""
    ###
    ofi=@obj_filepath.tr("/","\\")
    cline=cline+" -l \""+ofi+"\"" ### already linked when made but ?
    cline=cline+" -r \""+ofi+"\""
    ###
    if @resolution=="true"
      wi=@width
      he=@height
      cline=cline+" --film-width " +wi.to_s
      cline=cline+" --film-height "+he.to_s
    else
      wi=nil
      he=nil
    end#if
    ###
    ### ALWAYS export the camera !
  #if @export_camera=="true"
    ### get camera
    acc=scene.camera
    posa=acc.eye.to_a
    pos=[0,0,0]
    pos.x=posa.x.to_m.to_f
    pos.y=posa.z.to_m.to_f
    pos.z=posa.y.to_m.to_f * -1
    targeta=acc.target.to_a
    target=[0,0,]
    target.x=targeta.x.to_m.to_f
    target.y=targeta.z.to_m.to_f
    target.z=targeta.y.to_m.to_f * -1
    upa=acc.up.to_a
    up=[0,0,0]
    up.x=upa.x.to_m.to_f
    up.y=upa.z.to_m.to_f
    up.z=upa.y.to_m.to_f * -1
    ###
    cline=cline+" --cam-pos-x "+pos.x.to_s
    cline=cline+" --cam-pos-y "+pos.y.to_s
    cline=cline+" --cam-pos-z "+pos.z.to_s
    ###
    cline=cline+" --cam-target-x "+target.x.to_s
    cline=cline+" --cam-target-y "+target.y.to_s
    cline=cline+" --cam-target-z "+target.z.to_s
    ###
    cline=cline+" --cam-up-x "+up.x.to_s
    cline=cline+" --cam-up-y "+up.y.to_s
    cline=cline+" --cam-up-z "+up.z.to_s
    ###
    fol=acc.focal_length
    fovV=acc.fov
    wid=@width.to_f
    hei=@height.to_f
    if acc.image_width != 0
      wid=acc.image_width
      if acc.aspect_ratio != 0
        aro=acc.aspect_ratio
      else
        aro=wid/hei
      end#if
      hei=wid/aro
    end#if
    h=2*fol*Math::tan(fovV.degrees/2)
    w=h*wid/hei
    fov=(2*Math::atan(w/(2*fol))).radians
    cline=cline+" --cam-fov "+fov.to_s
  #end#if export_camera
    ###
    if @lens_aperture_radius=="true"
      apr=@lens_aperture_radius_value
      cline=cline+" --cam-aperture "+apr.to_s
    else
      apr=nil
    end#if
    if @focal_depth=="true"
      foc=@focal_depth_value
      cline=cline+" --cam-focaldepth "+foc.to_s
    end#if
    ###
    if @motion_blur=="true"
      mob=@motion_blur_value
    else
      mob=""
    end#if
    mbc=nil
    case mob
      when "Next"
        mbc=nil
        scene_indx=scenes.index(scene)
        mb_indx=scene_indx+1
        mbc=scenes[mb_indx]
      when "Previous"
        mbc=nil
        scene_indx=scenes.index(scene)
        mb_indx=scene_indx-1
        mbc=scenes[mb_indx]
      else
        mbc=nil
    end#case
    if mbc
      mposa=mbc.eye.to_a
      mpos=[0,0,0]
      mpos.x=mposa.x.to_m.to_f
      mpos.y=mposa.z.to_m.to_f
      mpos.z=mposa.y.to_m.to_f * -1
      mtargeta=mbc.target.to_a
      mtarget=[0,0,0]
      mtarget.x=mtargeta.x.to_m.to_f
      mtarget.y=mtargeta.z.to_m.to_f
      mtarget.z=mtargeta.y.to_m.to_f * -1
      mupa=mbc.up.to_a
      mup=[0,0,0]
      mup.x=mupa.x.to_m.to_f
      mup.y=mupa.z.to_m.to_f
      mup.z=mupa.y.to_m.to_f * -1
      ###
      cline=cline+" --cam-motion-pos-x "+mpos.x.to_s
      cline=cline+" --cam-motion-pos-y "+mpos.y.to_s
      cline=cline+" --cam-motion-pos-z "+mpos.z.to_s
      ###
      cline=cline+" --cam-motion-target-x "+mtarget.x.to_s
      cline=cline+" --cam-motion-target-y "+mtarget.y.to_s
      cline=cline+" --cam-motion-target-z "+mtarget.z.to_s
      ###
      cline=cline+" --cam-motion-up-x "+mup.x.to_s
      cline=cline+" --cam-motion-up-y "+mup.y.to_s
      cline=cline+" --cam-motion-up-z "+mup.z.to_s
      ###
    end#if
    ### sun etc ?
    if @export_sun_direction=="true"
      sunv=scene.shadow_info["SunDirection"]
      suna=sunv.to_a
      sun=[0,0,0]
      sun.x=suna.x.to_f
      sun.y=suna.z.to_f
      sun.z=suna.y.to_f * -1
      cline=cline+" --daylight-sundir-x "+sun.x.to_s
      cline=cline+" --daylight-sundir-y "+sun.y.to_s
      cline=cline+" --daylight-sundir-z "+sun.z.to_s
    end#if
    ### GPUs ?
    if @gpu_devices=="true"
      if @gpu_device0=="true"
        cline=cline+" -g 0"
      end#if
      if @gpu_device1=="true"
        cline=cline+" -g 1"
      end#if
      if @gpu_device2=="true"
        cline=cline+" -g 2"
      end#if
      if @gpu_device3=="true"
        cline=cline+" -g 3"
      end#if
    end#if
    cline=cline+" \""+ocs+"\"\n"
    ###
    opu.next! ### increments the img00001 > img00002 etc
    ###
  }#anim_scenes
    ###
    cline=cline+"@exit\n" if @pc
    ###
    ###
    if @pc
      ### make cmd file
      cmd_path=@project_path.tr("\\","/")+"/octane_anim.cmd"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
    else ### it's a Mac
      #UI.messagebox("Mac version is in beta testing...")
      cline.tr("\\","/")### force unix friendly separators
      cline.gsub(/\"/){'\\"'}### escape " as \"
      cmd_path=@project_path.tr("\\","/")+"/octane_anim.command"
      cmd=File.new(cmd_path, "w")
      cmd.puts cline
      cmd.close
      File.chmod(1411, cmd_path) ### 777 in octal is 1411 = execuatble
      ### run cmd file
      UI.openURL(cmd_path)
      #sleep(2); File.delete(cmd_path)
      ###delete cmd file afterwards: disabled for debug.
      ###
    end#if

    ### we must wait for frames to be made... or cancel HOW?
    
    if @pc
      sleep 2
      procs=self.processes_pc()
      while procs.include?("cmd.exe")
        #if @dlg.get_element_value("render_animation_button")=="Render Animation"
          #self.kill_process_pc("cmd.exe")
        #end#if
        sleep 1
        procs=self.processes_pc()
      end#wait till octane closed
      self.set_button()
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    else ### Mac
      sleep 2
      procs=self.processes_mac()
      while procs.include?("cmd.exe")
        #if @dlg.get_element_value("render_animation_button")=="Render Animation"
          #self.kill_process_mac("cmd.exe")
        #end#if
        sleep 1
        procs=self.processes_pc()
      end#wait till octane closed
      self.set_button()
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",false);")
      @dlg.execute_script(js)
    end#if
    
    ###
    Sketchup.set_status_text("Octane:")
    ###
  end#render_animation
  
  def kill_process_pc(proc)
    puts(proc+" killed.")
    ### add code here
  end#kill_process_pc
  
  def kill_process_mac(proc)
    puts(proc+" killed.")
    ### add code here
  end#kill_process_pc
  
  def processes_pc()
    return if not @pc
    require 'win32ole.so'
    procs=WIN32OLE.connect("winmgmts:\\\\.")
    ps=[]
    procs.InstancesOf("win32_process").each{|p|ps << p.name.to_s.downcase}
    return ps
  end#processes_pc
  
  def processes_mac()
    puts "Mac process list."
    return [] ### to add code here
  end#processes_mac
  
  def set_button(ren=false)
    if ren
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"value\",\"Rendering Animation, Please Wait...\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"value\",\"Render Animation\");")
      @dlg.execute_script(js)
    end#if
  end#set_button
  
  def is_cancelled?()
    
  end#is_cancelled?
  
  def help()
    help="OctaneHelp.pdf"
    help_path=File.join(File.dirname(__FILE__), help)
    if File.exist?(help_path)
      UI.openURL(help_path)
    else
      UI.messagebox("Octane:\nHelp file not found\n["+help_path+"]")
    end#if
  end#help
  
  def getIDvalues()
    @values={}
    @@idset.each{|id|@values[id]=@dlg.get_element_value(id)}
    ### set variables to these values too
    self.setIDvalues()
  end#getIDvalues
  
  def setIDvalues()
    @octane_render_binary=@values["octane_render_binary"]
	@project_path=@values["project_path"]
	@project_name=@values["project_name"]
	@native_unit_size=@values["native_unit_size"]
    
	@resolution=@values["resolution"]
    @width=@values["width"]
    @height=@values["height"]
    @percentage=@values["percentage"]
    
    @export_camera=@values["export_camera"]
    @active_camera=@values["active_camera"]
    @lens_aperture_radius=@values["lens_aperture_radius"]
    @lens_aperture_radius_value=@values["lens_aperture_radius_value"]
    @focal_depth=@values["focal_depth"]
    @focal_depth_value=@values["focal_depth_value"]
    @motion_blur=@values["motion_blur"]
    @motion_blur_value=@values["motion_blur_value"]
    
    @export_sun_direction=@values["export_sun_direction"]
    @light_source=@values["light_source"]
    
    @gpu_devices=@values["gpu_devices"]
    @gpu_device0=@values["gpu_device0"]
    @gpu_device1=@values["gpu_device1"]
    @gpu_device2=@values["gpu_device2"]
    @gpu_device3=@values["gpu_device3"]

    @image_output=@values["image_output"]
    @start_frame=@values["start_frame"]
    @end_frame=@values["end_frame"]
    @step=@values["step"]
    @pixel_samples=@values["pixel_samples"]
    @pixs=@values["pixs"]
  end#setIDvalues
  
  def saveIDvalues()
    #puts 'saveIDvalues TO DO'
    ### save all settings to SUp or SKP
    Sketchup.write_default("Octane","octane_render_binary", @octane_render_binary.gsub(/\\/){'\\\\'})
    #
    @model.set_attribute("Octane", "project_path", @project_path)
    @model.set_attribute("Octane", "project_name", @project_name)
	#@native_unit_size dynamic
    
    @model.set_attribute("Octane", "resolution", @resolution)
    @model.set_attribute("Octane", "width", @width)
    @model.set_attribute("Octane", "height", @height)
    @model.set_attribute("Octane", "percentage", @percentage)
    
    @model.set_attribute("Octane", "export_camera", @export_camera)
    @model.set_attribute("Octane", "active_camera", @active_camera)
    @model.set_attribute("Octane", "lens_aperture_radius", @lens_aperture_radius)
    @model.set_attribute("Octane", "lens_aperture_radius_value", @lens_aperture_radius_value)
    @model.set_attribute("Octane", "focal_depth", @focal_depth)
    @model.set_attribute("Octane", "focal_depth_value", @focal_depth_value)
    @model.set_attribute("Octane", "motion_blur", @motion_blur)
    @model.set_attribute("Octane", "motion_blur_value", @motion_blur_value)
    
    @model.set_attribute("Octane", "export_sun_direction", @export_sun_direction)
    @model.set_attribute("Octane", "light_source", @light_source)
    
    @model.set_attribute("Octane", "gpu_devices", @gpu_devices)
    @model.set_attribute("Octane", "gpu_device0", @gpu_device0)
    @model.set_attribute("Octane", "gpu_device1", @gpu_device1)
    @model.set_attribute("Octane", "gpu_device2", @gpu_device2)
    @model.set_attribute("Octane", "gpu_device3", @gpu_device3)

    @model.set_attribute("Octane", "start_frame", @start_frame)
    @model.set_attribute("Octane", "end_frame", @end_frame)
    @model.set_attribute("Octane", "step", @step)
    @model.set_attribute("Octane", "pixel_samples", @pixel_samples)
    @model.set_attribute("Octane", "pixs", @pixs)
    ###
  end#saveIDvalues
  
############## obj exporter part #############################
  
    def run_exporter(anim=false)
        self.settings(anim)
        self.do_export()
	end
	
	def settings(anim=false)
		@obj_="true"
        @obj_name=@project_name
        @obj_name=@obj_name+"_anim" if anim
        @obj_name=@obj_name+".obj"
        @obj_filepath=@project_path+"/"+@obj_name
        @mtllib=@project_name
        @mtllib=@mtllib+"_anim" if anim
        @mtllib=@mtllib+".mtl"
        @mtl_file=@project_path+"/"+@mtllib
		@textures_name=@project_name+"_Textures"
        @textures_name=@textures_name+"_anim" if anim
		@v_counter=0
		@obj_list=[]
		@textures_path=""
		@used_materials=[]
	end
    
	def do_export()
        Sketchup.set_status_text("Octane: Exporting OBJ & MTL Files...")
		starttime=Time.now.to_f
	    @title=Sketchup.active_model.title
	    @title="Untitled" if @title==""
	    Dir.mkdir(@project_path) if not File.exist?(@project_path)
        ###
        self.export_start()
        ###
		endtime=(((Time.now.to_f - starttime)*10000).to_i/10000.0).to_f.to_s
		Sketchup.set_status_text("Octane: OBJ & MTL Files Completed in #{endtime} seconds")
	end
    
    def export_start()
		@obj_file=File.new(@obj_filepath, "w")
        @obj_file.puts "# Alias Wavefront OBJ File Exported from SketchUp by 'Octane'"
        @obj_file.puts "# (c) 2010 TIG & Refractive Software"
        @obj_file.puts "# Units = meters"
        @obj_file.puts ""
		@obj_file.puts "mtllib #{@mtllib}"
        @obj_file.puts ""
		Sketchup.set_status_text("Octane: Making OBJ File...")
		self.export_obj()
		@obj_file.close if @obj_file
		Sketchup.set_status_text("Octane: Making MTL File...")
		self.export_mtl_material()
        Sketchup.set_status_text("Octane: Exporting Textures...")
		self.export_textures()
	end

	def export_obj()
		title=@title
		ents=Sketchup.active_model.entities.to_a
		objname=title+"_Main"
		Sketchup.set_status_text("Octane: Making Meshes...")
        faces=ents.find_all{|e|e.class==Sketchup::Face}
	    if faces[0]
			@obj_list << [objname, nil, "Default_Material"] 
		    self.export_entities(faces, objname)
		end#if
		Sketchup.set_status_text("Octane: Meshes Completed.")
		tr=Geom::Transformation.axes( (Geom::Point3d.new 0,0,0),(Geom::Vector3d.new(1,0,0)),(Geom::Vector3d.new(0,1,0)),(Geom::Vector3d.new(0,0,1)) )
        ###
	    ents.find_all{|e|e.class==Sketchup::Group}.each{|gp| self.export_group(gp, title, tr) }
	    ents.find_all{|e|e.class==Sketchup::ComponentInstance}.each{|ci| self.export_component_instance(ci, title, tr) }
        ###
	end

	def export_group(gp, title="", tr=Geom::Transformation.new, mat=nil)
		return if gp.hidden? == true or gp.layer.visible? == false
		ot=tr * gp.transformation
		defmat=mat if mat and gp.material == nil
		defmat=gp.material if gp.material
		@used_materials << defmat
		id=gp.entityID
		name="#{id}"
        name=name + "-" + gp.name if gp.name or gp.name=""
		name.gsub!(/[^\-_0-9A-Za-z]/,"_")
		objname=title + "_g_" + name
		@matname="Default_Material"
        if defmat
			@matname=defmat.display_name.gsub(/[^\-_0-9A-Za-z]/,'_')
		end
		if gp.entities.find{|e|e.class==Sketchup::Face}
			self.export_entities(gp.entities.find_all{|e|e.class==Sketchup::Face}, objname, ot, defmat) if @obj_ or @obj_list.find{|ol| ol[0] == objname } == nil
			@obj_list << [objname, ot, @matname]
		end
        ###
		gp.entities.find_all{|e| e.class==Sketchup::Group}.each{|f| self.export_group(f, title, ot, defmat) }
	    gp.entities.find_all{|e| e.class==Sketchup::ComponentInstance}.each{|f| self.export_component_instance(f, title, ot, defmat) }
        ###
	end
     
	def export_component_instance(ci, title="", tr=Geom::Transformation.new, mat=nil)
		return if ci.hidden? or not ci.layer.visible?
		ot=tr * ci.transformation
		defmat=mat if mat and ci.material == nil
		defmat=ci.material if ci.material
		@used_materials << defmat
		id=ci.entityID
		name=ci.definition.name
		name="#{id}" if name.length == 0
		name.gsub!(/[^\-_0-9A-Za-z]/,"_")
		objname=title + "_comp_" + name
		@matname="Default_Material"
        if defmat
			@matname=defmat.display_name.gsub(/[^\-_0-9A-Za-z]/,'_')
		end
		if ci.definition.entities.find{|e| e.class==Sketchup::Face }
			self.export_entities(ci.definition.entities.find_all{|e|e.class==Sketchup::Face}, objname, ot, defmat) if @obj_ or @obj_list.find{|ol| ol[0] == objname } == nil
			@obj_list << [objname, ot, @matname]
		end
		(ci.definition.entities.find_all{|e| e.class==Sketchup::Group}).each{|f| self.export_group(f, title, ot, defmat) }
	    (ci.definition.entities.find_all{|e| e.class==Sketchup::ComponentInstance}).each{|f| self.export_component_instance(f, title, ot, defmat) }
	end

	def export_entities(all_faces=[], objname="", tr=nil, defmat=nil)
	    mats=[nil]
	    Sketchup.active_model.materials.each{|mat| mats << mat }
		mats.each{|mat|
			faces=all_faces.find_all{|face|face.material==mat}
			if faces[0]
                @used_materials << mat
            else
                next
            end#if
            ###
            vs=[]
			nos=[]
			uvs=[]
			meshes=[]
			Sketchup.set_status_text("Octane: Prosessing Faces...")
            ###
			faces.each{|face|
				next if face.hidden? or not face.layer.visible?
				mesh=face.mesh(5)###7=backs too
				next if not mesh
                f_uvs=(1..mesh.count_points).map{|i|mesh.uv_at(i,1)}####1=front
                f_vs=[]
                f_vs=(1..mesh.count_points).map{|i|mesh.points[i-1]}
				f_nos=[]
                f_nos=(1..mesh.count_points).map{|i|mesh.normal_at(i)}
                f_vcount=1; f_vcount=vs.length + 1 if vs[0]
				meshes.concat(mesh.polygons.map{|p| [p.map{|px|(f_vs.index(mesh.points[(px.abs-1)]) + f_vcount)}] })
                ###
                vs.concat(f_vs) if f_vs
				uvs.concat(f_uvs) if f_uvs
				nos.concat(f_nos) if f_nos
                ###
			}#faces.each
            ###
            @vs=vs
			@nos=nos
			@uvs=uvs
			@meshes=meshes
            mat=defmat if not mat
			self.export_obj_file(objname, tr, mat)
            ######################################
		}#mats.each
	end
    
    def flattenUVQ(uvq) ### UNUSED ???
        return Geom::Point3d.new((uvq.x/uvq.z), (uvq.y/uvq.z), 1.0)
    end#flattenUVQ

	def export_obj_file(objname="", tr=nil, mat=nil)
		Sketchup.set_status_text("Octane: Making OBJ File...")
		ot=Geom::Transformation.new
		ot=tr if tr 
        if mat
			@matname=mat.display_name.gsub(/[^\-_0-9A-Za-z]/,'_')
        else
            @matname="Default_Material"
		end
		return if @meshes == nil
		objname.gsub!(/[^\-_0-9A-Za-z]/,"_")
		if @meshes.length != 0 and @vs.length != 0
			if @matname
				@obj_file.puts "g #{objname}-#{@matname}"
				@obj_file.puts "usemtl #{@matname}"
			else
				@obj_file.puts "g #{objname}-Default_Material"
				@obj_file.puts "usemtl Default_Material"
				@matname="Default_Material"
			end#if
		    @v_counter=0 if not @v_counter
			@vs.each{|v|
				v=ot * v
				@obj_file.puts "v #{"%.6f" % v.x.to_m.to_f} #{"%.6f" % v.z.to_m.to_f} #{"%.6f" % (-v.y.to_m.to_f)}"
			}
			@nos.each{|vnor|
				nor=ot * vnor
				nor.normalize!
				@obj_file.puts "vn #{"%.6f" % (nor.x)} #{"%.6f" % (nor.z)} #{"%.6f" % (-nor.y)}"
			}
			@uvs.each{|uv|
				@obj_file.puts "vt #{"%.6f" % (uv.x)} #{"%.6f" % (uv.y)}"
			}
			@meshes.each{|mesh|
				f_str="f"
				mesh.each do |pg|
					pg.each{|j|
						k=j+@v_counter
						f_str += " #{k}/#{k}/#{k}"
					}
					@obj_file.puts f_str
				end
			}
			@obj_file.puts ""
			@v_counter += @vs.length if @vs[0]
		end
		@meshes=nil
		@uvs=nil
		@nos=nil
		@vts=nil
        Sketchup.set_status_text("Octane: OBJ File Completed.")
	end

	def export_mtl_material()
        Sketchup.set_status_text("Octane: Making MTL File...")
		mtl_file=File.new(@mtl_file,"w")
        mtl_file.puts "# Alias Wavefront MTL File Exported from SketchUp by 'Octane'"
        mtl_file.puts "# (c) 2010 TIG & Refractive Software"
        mtl_file.puts "# Made for '"+@obj_name+"'"
        mtl_file.puts ""
		mtl_file.puts "newmtl Default_Material"
		mtl_file.puts "Ka 0.300000 0.300000 0.300000"
		mtl_file.puts "Kd 0.700000 0.700000 0.700000"
		mtl_file.puts "Ks 0.000000 0.000000 0.000000"
        mtl_file.puts "d 1.000000"
		mtl_file.puts ""
        saved_names=[]
		@used_materials.uniq!
		@used_materials.each{|mat|
			next if not mat
            mat_name=mat.display_name.gsub(/[^\-_0-9A-Za-z]/,'_')
            saved_name=File.basename(mat_name)
			saved_name_uniq=self.make_name_unique(saved_names, saved_name)
			saved_names << saved_name_uniq
            @matname=saved_name_uniq
			if mat and mat.texture
                texture_extn=File.extname(mat.texture.filename)
				texture_path=@textures_name + "/" + mat_name + texture_extn
			end
			mtl_file.puts "newmtl #{mat_name}"
			if not mat.use_alpha?
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.330000 0.330000 0.330000"
				mtl_file.puts "d 1.000000"
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			else ### it's transparent
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.330000 0.330000 0.330000"
				mtl_file.puts "d #{"%.6f" % mat.alpha }"### it's NOT (1 - mat.alpha) !!!
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			end#if
            mtl_file.puts ""
		}
		mtl_file.puts "#EOF"
		mtl_file.flush
		mtl_file.close
        Sketchup.set_status_text("Octane: MTL File Completed.")
	end

	def make_texture_folder()
		@textures_path=@project_path+"/"+@textures_name if @textures_name != ""
		Dir.mkdir(@textures_path) if not File.exist?(@textures_path)
	end
    
	def export_textures()
        Sketchup.set_status_text("Octane: Exporting Textures...")
		self.make_texture_folder()
		temp_group=Sketchup.active_model.active_entities.add_group()
		saved_names=[]
        processed_textures=[]
		texture_names_by_material=Hash.new
        tw=Sketchup.create_texture_writer
		Sketchup.active_model.materials.each{|mat|
			if mat and mat.texture
                next if not @used_materials.include?(mat)
                ### don't export unused / visible materials for speed
				temp_group.material=mat
                mat_texture_file=mat.texture.filename.tr("\\","/")
                mat_texture_extn=File.extname(mat_texture_file)
                mat_base_bname=File.basename(mat_texture_file)
                mat_texture_basename=File.basename(mat_texture_file, mat_texture_extn)
                mat_texture_name=mat.display_name.gsub(/[^\-_0-9A-Za-z]/,'_') + mat_texture_extn
                saved_name=mat_texture_name
				saved_name_uniq=self.make_name_unique(saved_names, saved_name)
				saved_names << saved_name_uniq
                mat_texture_basename_next=mat_texture_basename+"1"
                if processed_textures.include?(mat_base_bname)
                  mat_base_bname=mat_texture_basename_next + mat_texture_extn
                  while processed_textures.include?(mat_base_bname)
                    mat_texture_basename_next.next!
                  end#while
                  mat_base_bname=mat_texture_basename_next + mat_texture_extn
                end#if
                processed_textures<< mat_base_bname
                texture_names_by_material[mat_base_bname]=saved_name_uniq
                tw.load(temp_group, true)
			end#if
		}
        tw.write_all(@textures_path, false)
        ###
		temp_group.erase! if temp_group.valid?
        ### rename textures to suit
        texture_names_by_material.each{|a,b|File.rename(File.join(@textures_path, a), File.join(@textures_path, b))}
        ###
		Sketchup.set_status_text("Octane: Textures Exported.")
	end
    
	def make_name_unique(saved_names=[], saved_name="")
		if saved_names.include?(saved_name)
			counter=1
			while counter < 1000
				new_name=File.basename(saved_name, ".*") + counter.to_s + File.extname(saved_name)
				return new_name if not saved_names.include?(new_name)
				counter += 1
			end
		end
		return saved_name
	end
############## end of exporter code #########################
  
end#class Octane ##########################################################
###

### make shortcut to tool #################################################
def octane()
  Sketchup.active_model.select_tool(Octane.new)
end#def octane
###


### add menu item #########################################################
if not file_loaded?(File.basename(__FILE__))
  UI.menu("File").add_item("Octane..."){octane}
  file_loaded(File.basename(__FILE__))
end#if
###